// Todo Still under development

import 'dart:async';

import 'package:camerax/camerax.dart' as cam;
import 'package:flutter/cupertino.dart';
import 'package:permission_handler/permission_handler.dart';

/// Camera
class Camera extends cam.CameraView {
  Camera({
    Key? key,
    required CameraController controller,
  }) : super(controller._controller);

  Camera.scanner({
    Key? key,
    required CameraController controller,
  }) : super(controller._controller);
}

class CameraController {
  cam.CameraController _controller;
  Function(PermissionStatus)? onPermission;
  final CameraFacing cameraFacing;

  CameraController([this.cameraFacing = CameraFacing.back]) : _controller = cam.CameraController(cameraFacing.get) {
    Permission.camera.request().then((status) => (onPermission ?? (s) {})(status));
  }

  set torchState(CameraTorchState torchState) => _controller.torchState.value = torchState.get;
  set texture(int textureId) => _controller.args.value = cam.CameraArgs(textureId, _controller.args.value!.size);
  set size(Size size) => _controller.args.value = cam.CameraArgs(_controller.args.value!.textureId, size);
  Stream<cam.Barcode> get barcodes => _controller.barcodes;

  Future<void> startAsync() => _controller.startAsync();

  StreamSubscription subscribeToBarcodes({
    CameraCodeFormat? format,
    Function(cam.Barcode barcode)? detection,
  }) {
    return _controller.barcodes.listen((barcode) {
      if (format == null || format.get == barcode.format) {
        if (detection != null) detection(barcode);
      }
    });
  }

  void dispose() => _controller.dispose();
  void torch() => _controller.torch();
}

enum CameraFacing { front, back }

extension on CameraFacing {
  cam.CameraFacing get get {
    switch (this) {
      case CameraFacing.back:
        return cam.CameraFacing.back;
      case CameraFacing.front:
      default:
        return cam.CameraFacing.front;
    }
  }
}

enum CameraTorchState { on, off, auto }

extension on CameraTorchState {
  cam.TorchState get get {
    switch (this) {
      case CameraTorchState.on:
        return cam.TorchState.on;
      default:
        return cam.TorchState.off;
    }
  }
}

enum CameraCodeFormat { all, code128, code39, code93, codebar, data_matrix, ean13, ean8, itf, qr_code, upc_a, upc_e, pdf417, asztec }

extension on CameraCodeFormat {
  cam.BarcodeFormat get get {
    switch (this) {
      case CameraCodeFormat.all:
        return cam.BarcodeFormat.all;
      case CameraCodeFormat.asztec:
        return cam.BarcodeFormat.aztec;
      case CameraCodeFormat.codebar:
        return cam.BarcodeFormat.codebar;
      case CameraCodeFormat.code39:
        return cam.BarcodeFormat.code39;
      case CameraCodeFormat.code93:
        return cam.BarcodeFormat.code93;
      case CameraCodeFormat.code128:
        return cam.BarcodeFormat.code128;
      case CameraCodeFormat.data_matrix:
        return cam.BarcodeFormat.data_matrix;
      case CameraCodeFormat.ean8:
        return cam.BarcodeFormat.ean8;
      case CameraCodeFormat.ean13:
        return cam.BarcodeFormat.ean13;
      case CameraCodeFormat.itf:
        return cam.BarcodeFormat.itf;
      case CameraCodeFormat.pdf417:
        return cam.BarcodeFormat.pdf417;
      case CameraCodeFormat.qr_code:
        return cam.BarcodeFormat.qr_code;
      case CameraCodeFormat.upc_a:
        return cam.BarcodeFormat.upc_a;
      case CameraCodeFormat.upc_e:
        return cam.BarcodeFormat.upc_e;
      default:
        return cam.BarcodeFormat.all;
    }
  }
}
