import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nueli/nueli.dart';

/// Pager2
class Pager2 extends Struct {
  final Ref<TabController> _tabController = $();
  final Ref<PageController> _pageController = $();

  final List<Page2> sections;
  final PageController controller;
  final Decor? decor;
  final Decor? indicator;
  final Duration duration;
  final Curve curve;

  Pager2({
    this.sections = const [],
    required this.controller,
    this.decor,
    this.indicator,
    this.duration = const Duration(milliseconds: 500),
    this.curve = Curves.linearToEaseOut,
  });

  Widget _header(BuildContext context) {
    goTo(int page) => _pageController.data!.animateToPage(page, duration: duration, curve: curve);

    final tabs = sections.map((e) => Tab(child: e.header)).toList();
    return SizedBox(
      child: TabBar(
        tabs: tabs,
        onTap: (i) => goTo(i),
        controller: _tabController.data,
        physics: AlwaysScrollableScrollPhysics(parent: BouncingScrollPhysics()),
        isScrollable: sections.length > 3,
        indicatorSize: TabBarIndicatorSize.label,
        automaticIndicatorColorAdjustment: true,
        labelPadding: EdgeInsets.symmetric(vertical: 8),
        overlayColor: MaterialStateProperty.all<Color>(palette.accent.withAlpha(50)),
        indicator: (indicator ?? $decor.color(palette.accent.withAlpha(25)).stroke(0.5, palette.accent.withAlpha(100))).toDecoration,
      ),
    );
  }

  Widget _pageView(BuildContext context) {
    return PageView(
      children: sections.map((e) => e.body).toList(),
      controller: _pageController.data,
      allowImplicitScrolling: true,
      onPageChanged: (i) => _tabController.data!.animateTo(i, curve: curve, duration: duration),
      physics: AlwaysScrollableScrollPhysics(parent: BouncingScrollPhysics()),
      clipBehavior: Clip.none,
    );
  }

  @override
  Widget get body => Stack(children: [
        (decor ?? $decor)(),
        _pageView(context),
        _header(context),
      ]);

  @override
  StructState<Struct> createState() => _Pager2State();
}

class _Pager2State<T extends Pager2> extends StructState<T> {
  @override
  void dispose() {
    widget._pageController.data!.dispose();
    super.dispose();
  }
}

class Page2 {
  final Widget header;
  final Widget body;
  final Decor? decor;

  Page2({
    Widget header = const SizedBox(),
    required Widget body,
    this.decor,
    Decor? headerDecor,
    Decor? bodyDecor,
  })  : header = (headerDecor ?? $decor).withChild(header)(),
        body = (bodyDecor ?? $decor).withChild(body)();

  Page2.textHeader({
    Txt? title,
    Decor? titleDecor,
    required this.body,
    this.decor,
  }) : header = (titleDecor ?? $decor).withChild((title ?? $txt)())();
}
